@@ -6,25 +6,31 @@ import base64
 PY_MAJOR = sys.version_info[0]
 PY2 = PY_MAJOR == 2
 PY3 = PY_MAJOR == 3
+PY3_ORDERED_DICT = PY3 and sys.version_info[1] >= 6  # Python 3.6+
 
-class_types = type,
+class_types = (type,)
+iterator_types = (type(iter('')),)
 
 if PY3:
-    import queue
     import builtins
+    import queue
+    from base64 import encodebytes, decodebytes
+    from collections.abc import Iterator as abc_iterator
+
     string_types = (str,)
     numeric_types = (int, float)
     ustr = str
-    from base64 import encodebytes, decodebytes
 else:
-    queue = __import__('Queue')
+    from collections import Iterator as abc_iterator  # noqa
+
     builtins = __import__('__builtin__')
-    string_types = (builtins.basestring,)
-    numeric_types = (int, float, builtins.long)
-    ustr = builtins.unicode
+    class_types += (types.ClassType,)
     encodebytes = base64.encodestring
     decodebytes = base64.decodestring
-    class_types += types.ClassType,
+    string_types = (builtins.basestring,)
+    numeric_types = (int, float, builtins.long)
+    queue = __import__('Queue')
+    ustr = builtins.unicode
 
 
 def iterator(class_):
