@@ -16,8 +16,9 @@ from .compat import numeric_types
 from .backend import json
 
 
-def decode(string, backend=None, context=None, keys=False, reset=True,
-           safe=False, classes=None):
+def decode(
+    string, backend=None, context=None, keys=False, reset=True, safe=False, classes=None
+):
     """Convert a JSON string into a Python object.
 
     The keyword argument 'keys' defaults to False.
@@ -50,6 +51,11 @@ def _safe_hasattr(obj, attr):
         return False
 
 
+def _is_json_key(key):
+    """Has this key a special object that has been encoded to JSON?"""
+    return isinstance(key, compat.string_types) and key.startswith(tags.JSON_KEY)
+
+
 class _Proxy(object):
     """Proxies are dummy objects that are later replaced by real instances
 
@@ -78,6 +84,7 @@ class _Proxy(object):
     when swapping proxies with real instances.
 
     """
+
     def __init__(self):
         self.instance = None
 
@@ -106,7 +113,6 @@ def _obj_setvalue(obj, idx, proxy):
 
 
 class Unpickler(object):
-
     def __init__(self, backend=None, keys=False, safe=False):
         self.backend = backend or json
         self.keys = keys
@@ -202,8 +208,10 @@ class Unpickler(object):
         elif util.is_dictionary(obj):
             restore = self._restore_dict
         else:
+
             def restore(x):
                 return x
+
         return restore(obj)
 
     def _restore_base64(self, obj):
@@ -353,8 +361,7 @@ class Unpickler(object):
         if kwargs:
             kwargs = self._restore(kwargs)
 
-        is_oldstyle = (
-            not (isinstance(cls, type) or getattr(cls, '__meta__', None)))
+        is_oldstyle = not (isinstance(cls, type) or getattr(cls, '__meta__', None))
         try:
             if (not is_oldstyle) and hasattr(cls, '__new__'):
                 # new style classes
@@ -385,8 +392,9 @@ class Unpickler(object):
 
         instance = self._restore_object_instance_variables(obj, instance)
 
-        if (_safe_hasattr(instance, 'default_factory') and
-                isinstance(instance.default_factory, _Proxy)):
+        if _safe_hasattr(instance, 'default_factory') and isinstance(
+            instance.default_factory, _Proxy
+        ):
             instance.default_factory = instance.default_factory.get()
 
         return instance
@@ -396,7 +404,7 @@ class Unpickler(object):
         method = _obj_setattr
         deferred = {}
 
-        for k, v in sorted(obj.items(), key=util.itemgetter):
+        for k, v in util.items(obj):
             # ignore the reserved attribute
             if ignorereserved and k in tags.RESERVED:
                 continue
@@ -408,8 +416,7 @@ class Unpickler(object):
             k = restore_key(k)
             # step into the namespace
             value = self._restore(v)
-            if (util.is_noncomplex(instance) or
-                    util.is_dictionary_subclass(instance)):
+            if util.is_noncomplex(instance) or util.is_dictionary_subclass(instance):
                 try:
                     if k == '__dict__':
                         setattr(instance, k, value)
@@ -446,7 +453,7 @@ class Unpickler(object):
             if hasattr(instance, 'append'):
                 for v in obj[tags.SEQ]:
                     instance.append(self._restore(v))
-            if hasattr(instance, 'add'):
+            elif hasattr(instance, 'add'):
                 for v in obj[tags.SEQ]:
                     instance.add(self._restore(v))
 
@@ -457,8 +464,9 @@ class Unpickler(object):
 
     def _restore_state(self, obj, instance):
         state = self._restore(obj[tags.STATE])
-        has_slots = (isinstance(state, tuple) and len(state) == 2
-                     and isinstance(state[1], dict))
+        has_slots = (
+            isinstance(state, tuple) and len(state) == 2 and isinstance(state[1], dict)
+        )
         has_slots_and_dict = has_slots and isinstance(state[0], dict)
         if hasattr(instance, '__setstate__'):
             instance.__setstate__(state)
@@ -466,16 +474,16 @@ class Unpickler(object):
             # implements described default handling
             # of state for object with instance dict
             # and no slots
-            instance = self._restore_from_dict(
-                    state, instance, ignorereserved=False)
+            instance = self._restore_from_dict(state, instance, ignorereserved=False)
         elif has_slots:
-            instance = self._restore_from_dict(
-                    state[1], instance, ignorereserved=False)
+            instance = self._restore_from_dict(state[1], instance, ignorereserved=False)
             if has_slots_and_dict:
                 instance = self._restore_from_dict(
-                        state[0], instance, ignorereserved=False)
-        elif (not hasattr(instance, '__getnewargs__')
-              and not hasattr(instance, '__getnewargs_ex__')):
+                    state[0], instance, ignorereserved=False
+                )
+        elif not hasattr(instance, '__getnewargs__') and not hasattr(
+            instance, '__getnewargs_ex__'
+        ):
             # __setstate__ is not implemented so that means that the best
             # we can do is return the result of __getstate__() rather than
             # return an empty shell of an object.
@@ -505,20 +513,48 @@ class Unpickler(object):
 
     def _restore_dict(self, obj):
         data = {}
-        restore_key = self._restore_key_fn()
-        for k, v in sorted(obj.items(), key=util.itemgetter):
-            if isinstance(k, numeric_types):
-                str_k = k.__str__()
-            else:
-                str_k = k
-            self._namestack.append(str_k)
-            k = restore_key(k)
-            data[k] = self._restore(v)
-            # k is currently a proxy and must be replaced
-            if isinstance(data[k], _Proxy):
-                self._proxies.append((data, k, data[k], _obj_setvalue))
 
-            self._namestack.pop()
+        # If we are decoding dicts that can have non-string keys then we
+        # need to do a two-phase decode where the non-string keys are
+        # processed last.  This ensures a deterministic order when
+        # assigning object IDs for references.
+        if self.keys:
+            # Phase 1: regular non-special keys.
+            for k, v in util.items(obj):
+                if _is_json_key(k):
+                    continue
+                if isinstance(k, numeric_types):
+                    str_k = k.__str__()
+                else:
+                    str_k = k
+                self._namestack.append(str_k)
+                data[k] = self._restore(v)
+
+                self._namestack.pop()
+
+            # Phase 2: object keys only.
+            for k, v in util.items(obj):
+                if not _is_json_key(k):
+                    continue
+                self._namestack.append(k)
+
+                k = self._restore_pickled_key(k)
+                data[k] = result = self._restore(v)
+                # k is currently a proxy and must be replaced
+                if isinstance(result, _Proxy):
+                    self._proxies.append((data, k, result, _obj_setvalue))
+
+                self._namestack.pop()
+        else:
+            # No special keys, thus we don't need to restore the keys either.
+            for k, v in util.items(obj):
+                if isinstance(k, numeric_types):
+                    str_k = k.__str__()
+                else:
+                    str_k = k
+                self._namestack.append(str_k)
+                data[k] = self._restore(v)
+                self._namestack.pop()
         return data
 
     def _restore_key_fn(self):
@@ -536,17 +572,22 @@ class Unpickler(object):
         if self.keys:
             restore_key = self._restore_pickled_key
         else:
+
             def restore_key(key):
                 return key
+
         return restore_key
 
     def _restore_pickled_key(self, key):
         """Restore a possibly pickled key"""
-        if (isinstance(key, compat.string_types) and
-                key.startswith(tags.JSON_KEY)):
-            key = decode(key[len(tags.JSON_KEY):],
-                         backend=self.backend, context=self,
-                         keys=True, reset=False)
+        if _is_json_key(key):
+            key = decode(
+                key[len(tags.JSON_KEY) :],
+                backend=self.backend,
+                context=self,
+                keys=True,
+                reset=False,
+            )
         return key
 
     def _refname(self):
@@ -621,7 +662,7 @@ def loadclass(module_and_name, classes=None):
     # First assume that everything up to the last dot is the module name,
     # then try other splits to handle classes that are defined within
     # classes
-    for up_to in range(len(names)-1, 0, -1):
+    for up_to in range(len(names) - 1, 0, -1):
         module = util.untranslate_module_name('.'.join(names[:up_to]))
         try:
             __import__(module)
